@@ -82,11 +82,6 @@ class KcmContainer : public QScrollArea
         */
         const KAboutData *kcmAboutData() const;
 
-        /**
-        * Get currently selected KCMs export information
-        */
-        QString exportText() const;
-
         /**
         * Get currently selected KCMs name
         */
